#include <stdio.h>
#include <avr\io.h>
#define F_CPU 32000000UL
#include <util\delay.h>
#include <inttypes.h>
#include "xplSDRAM.h"

void Config32MHzClock(void);


#define TRUE ( 1==1)
#define FALSE ( 1!=0 )

/********************** initialization **************************************************/
void Config32MHzClock(void)
{
  CCP = CCP_IOREG_gc; //Security Signature to modify clock 
  // initialize clock source to be 32MHz internal oscillator (no PLL)
  OSC.CTRL = OSC_RC32MEN_bm; // enable internal 32MHz oscillator
  while(!(OSC.STATUS & OSC_RC32MRDY_bm)); // wait for oscillator ready
  CCP = CCP_IOREG_gc; //Security Signature to modify clock 
  CLK.CTRL = 0x01; //select sysclock 32MHz osc
};

void init(void)
{
  Config32MHzClock();

  CLK.PSCTRL = 0x00; // no division on peripheral clock

  //PORTCFG.CLKEVOUT = PORTCFG_CLKOUT_PE7_gc;
  //PORTE.DIR = (1<<7); // clkout
  PORTE.DIR = 0xFF; // led port
  // configure PORTF, USARTC0 (PORTF:3=Tx, PORTF:2=Rx) as asynch serial port
  // This will connect to the USB-Serial chip on EVAL-USB boards
  // For other boards rewrite all occurences of USARTC0 below with USARTE0
  // then you can use PORTE:2,3 as asynch serial port (EVAL-01, EVAL-04 boards)
  PORTC.DIR |= (1<<3) | (1<<0); // set PORTF:3 transmit pin as output
  PORTC.OUT |= (1<<3);          // set PORTF:3 hi 
  USARTC0.BAUDCTRLA = 207; // 9600b  (BSCALE=207,BSEL=0)
//  USARTC0.BAUDCTRLA = 103; // 19200b  (BSCALE=103,BSEL=0)
//  USARTC0.BAUDCTRLA = 34;  // 57600b  (BSCALE=34,BSEL=0)
//  USARTC0.BAUDCTRLA = 33; USARTC0.BAUDCTRLB = (-1<<4); // 115.2kb (BSCALE=33,BSEL=-1)
//  USARTC0.BAUDCTRLA = 31; USARTC0.BAUDCTRLB = (-2<<4); // 230.4kb (BSCALE=31,BSEL=-2)
//  USARTC0.BAUDCTRLA = 27; USARTC0.BAUDCTRLB = (-3<<4); // 460.8kb (BSCALE=27,BSEL=-3)
//  USARTC0.BAUDCTRLA = 19; USARTC0.BAUDCTRLB = (-4<<4); // 921.6kb (BSCALE=19,BSEL=-4)
//  USARTC0.BAUDCTRLA = 1; USARTC0.BAUDCTRLB = (1<<4); // 500kb (BSCALE=19,BSEL=-4)
//  USARTC0.BAUDCTRLA = 1;   // 1Mb (BSCALE=1,BSEL=0)

  USARTC0.CTRLB = USART_TXEN_bm | USART_RXEN_bm; // enable tx and rx on USART
	PORTE.OUT ^= 0xFF;
  
}
/********************** serial communication ********************************************/
void chPutChar(uint8_t zeichen)
{
    USARTC0.DATA = zeichen; // transmit ascii 3 over and over
	if(!(USARTC0.STATUS&USART_DREIF_bm))
		while(!(USARTC0.STATUS & USART_TXCIF_bm)); // wait for TX complete
  	USARTC0.STATUS |= USART_TXCIF_bm;  // clear TX interrupt flag
}
uint8_t chGetChar()
{

	while(!(USARTC0.STATUS&USART_RXCIF_bm));  // wait for RX complete
	PORTE.OUTTGL = (1<<7); // toogle indicator led

  	return USARTC0.DATA;
}
uint8_t chCharReady()
{
	return (USARTC0.STATUS&USART_RXCIF_bm);
}

void printString(char *data)
{
	unsigned char i = 0;
	while(data[i]!=0x00) chPutChar(data[i++]);
}

void PrintInt(int16_t wert)
{  	
	char text[6];
	itoa(wert,text,10);
	printString(text);
}
void PrintTextInt(char *str,int16_t wert)
{  	
	char text[6];
	printString(str);
	itoa(wert,text,10);
	printString(text);
	printString("  ");
	//printString("\n\r");
}

/********************** main programm ***************************************************/

int main(void)
{
  int data;
  uint8_t n;
  uint32_t adr;
  uint8_t wert;

  init();

  printString("XPLAIN RAM TEST");
  
  data='0';

  initSDRAM(); 

  while(1)
  {

	adr=0;
	chGetChar();
	printString("\n\r Schreib/Lese Test \n\r \n\r");
	for(n=0;n<8;n++)
	{
		PrintTextInt("Adresse*1000: ",adr);
		wert=adr+12;
		PrintTextInt("Wert schreiben: ",wert);
		writeFarSDRAM(adr*1000,wert);
		PrintTextInt("Wert lesen: ",readFarSDRAM(adr*1000));
		printString("\n\r");
	
		chGetChar();
	
		adr+=1000;
	}
	adr=0;

	printString("\n\r alle wieder lesen \n\r \n\r");
	for(n=0;n<8;n++)
	{
		PrintTextInt("Adresse*1000: ",adr);
		wert=adr+12;

		PrintTextInt("gelesener Wert: ",readFarSDRAM(adr*1000));
		printString("\n\r");
		
		adr+=1000;
		_delay_ms(1000);
	}

	printString("Geschwindigkeitstest: 1MByte schreiben \n\r");
	printString("bitte Taste druecken \n\r");
	chGetChar();
	printString("START !! \n\r");
	for(adr=0;adr<1000000UL;adr++)
		writeFarSDRAM(adr*1000,0xAA);
	printString("STOP: 1000000 Bytes geschrieben!! \n\r");

	printString("Geschwindigkeitstest: 1MByte lesen \n\r");
	printString("bitte Taste druecken \n\r");
	chGetChar();
	printString("START !! \n\r");
	for(adr=0;adr<1000000UL;adr++)
		readFarSDRAM(adr*1000);
	printString("STOP: 1000000 Bytes gelesen!! \n\r");

  }
}


